@@ -1,6 +1,6 @@
 /* 
 
-$Id: md5.c,v 1.1 2003/03/18 05:00:04 calrissian Exp $
+$Id: md5.c,v 1.2 2004/07/25 17:13:03 calrissian Exp $
 
 MD5C.C - RSA Data Security, Inc., MD5 message-digest algorithm
 
@@ -26,6 +26,9 @@ These notices must be retained in any copies of any part of this
 documentation and/or software.
  */
 
+#include <config.h>
+
+#ifndef HAVE_OPENSSL_MD5_H
 
 #include <string.h>
 #include "md5global.h"
@@ -355,3 +358,4 @@ unsigned int len;
 #endif
 }
 
+#endif
