@@ -1,7 +1,7 @@
 /*
- * $Id: shoot.h,v 1.1 2003/04/04 02:12:18 calrissian Exp $
+ * $Id: shoot.h,v 1.3 2004/06/13 19:35:26 calrissian Exp $
  *
- * Copyright (C) 2002-2003 Fhg Fokus
+ * Copyright (C) 2002-2004 Fhg Fokus
  *
  * This file belongs to sipsak, a free sip testing tool.
  *
@@ -19,6 +19,16 @@
 #ifndef SIPSAK_SHOOT_H
 #define SIPSAK_SHOOT_H
 
+#include "config.h"
+
+#ifdef HAVE_NETINET_IP_H
+#ifdef HAVE_NETINET_IP_ICMP_H
+#ifdef HAVE_NETINET_UDP_H
+#define RAW_SUPPORT
+#endif
+#endif
+#endif
+
 void shoot(char *buff);
 
 #endif
