@@ -1,5 +1,5 @@
 /*
- * $Id: sipsak.c,v 1.71 2004/08/04 21:40:18 calrissian Exp $
+ * $Id: sipsak.c,v 1.76 2004/10/08 18:07:14 calrissian Exp $
  *
  * Copyright (C) 2002-2004 Fhg Fokus
  * Copyright (C) 2004 Nils Ohlmeier
@@ -35,6 +35,8 @@
 #include <stdio.h>
 #include <string.h>
 #include <time.h>
+#include <regex.h>
+#include <sys/wait.h>
 
 #include "config.h"
 
@@ -47,13 +49,26 @@
 #include "shoot.h"
 #include "exit_code.h"
 
+/* regular expression for matching received replies */
+regex_t* re;
+
+
+static void sigchld_handler(int signo)
+{
+	int chld_status;
+	pid_t chld;
+
+	while ((chld = waitpid(-1, &chld_status, WNOHANG)) > 0);
+}
+
+
 void print_version() {
 	printf("%s %s by Nils Ohlmeier\n", PACKAGE_NAME, PACKAGE_VERSION);
 	printf(" Copyright (C) 2002-2004 FhG Fokus\n");
 	printf(" Copyright (C) 2004 Nils Ohlmeier\n");
 	printf(" report bugs to %s\n\n", PACKAGE_BUGREPORT);
 	printf(
-		" shoot  : sipsak [-f FILE] -s SIPURI\n"
+		" shoot  : sipsak [-f FILE] [-L] -s SIPURI\n"
 		" trace  : sipsak -T -s SIPURI\n"
 		" usrloc : sipsak -U [-I|M] [-b NUMBER] [-e NUMBER] [-x NUMBER] [-z] -s SIPURI\n"
 		" usrloc : sipsak -I|M [-b NUMBER] [-e NUMBER] -s SIPURI\n"
@@ -62,9 +77,11 @@ void print_version() {
 		" flood  : sipsak -F [-c NUMBER] -s SIPURI\n"
 		" random : sipsak -R [-t NUMBER] -s SIPURI\n\n"
 		" additional parameter in every mode:\n"
-		"   [-a PASSWORD] [-d] [-i] [-H HOSTNAME] [-l PORT] [-m NUMBER] [-n] "
+		);
+	printf("   [-a PASSWORD] [-d] [-i] [-H HOSTNAME] [-l PORT] [-m NUMBER] [-n] "
 			"[-N]\n"
-		"   [-r PORT] [-v] [-V] [-w]\n\n");
+		"   [-r PORT] [-v] [-V] [-w]\n\n"
+		);
 }
 
 void print_long_help() {
@@ -73,17 +90,21 @@ void print_long_help() {
 		"  --help                     displays this help message\n"
 		"  --version                  prints version string only\n"
 		"  --filename=FILE            the file which contains the SIP message to send\n"
+		"                               use - for standard input\n"
+		"  --no-crlf                  de-activate CR (\\r) insertion\n"
 		"  --sip-uri=SIPURI           the destination server uri in form\n"
 		"                               sip:[user@]servername[:port]\n"
 		"  --traceroute               activates the traceroute mode\n"
-		"  --usrloc-mode              activates the usrloc mode\n"
+		);
+	printf("  --usrloc-mode              activates the usrloc mode\n"
 		"  --invite-mode              simulates a successful calls with itself\n"
 		"  --message-mode             sends messages to itself\n"
 		"  --contact=SIPURI           use the given uri as Contact in REGISTER\n"
 		"  --appendix-begin=NUMBER    the starting number appendix to the user name (default: 0)\n"
 		"  --appendix-end=NUMBER      the ending numer of the appendix to the user name\n"
 		"  --sleep=NUMBER             sleep number ms before sending next request\n"
-		"  --expires=NUMBER           the expires header field value (default: 15)\n"
+		);
+	printf("  --expires=NUMBER           the expires header field value (default: 15)\n"
 		"  --remove-bindings          activates randomly removing of user bindings\n"
 		"  --flood-mode               activates the flood mode\n"
 		"  --cseq-max=NUMBER          the maximum CSeq number for flood mode "
@@ -91,13 +112,17 @@ void print_long_help() {
 		"  --random-mode              activates the random modues (dangerous)\n"
 		"  --trash-chars=NUMBER       the maximum number of trashed character in random mode\n"
 		"                               (default: request length)\n"
-		"  --local-port=PORT          the local port to use (default: any)\n"
+		);
+	printf("  --local-port=PORT          the local port to use (default: any)\n"
 		"  --remote-port=PORT         the remote port to use (default: 5060)\n"
 		"  --outbound-proxy=HOSTNAME  request target (outbound proxy)\n"
 		"  --hostname=HOSTNAME        overwrites the local hostname in all headers\n"
 		"  --max-forwards=NUMBER      the value for the max-forwards header field\n"
-		"  --numeric                  use IPs instead of FQDN in the Via-Line\n"
-		"  --no-via                   deactivate the insertion of a Via-Line\n"
+		"  --numeric                  use IPs instead of FQDN in the Via-Line\n");
+	printf("  --processes=NUMBER         Divide the workflow among the number of processes\n"
+		"  --auth-username=STRING     username for authentication\n"
+		);
+	printf("  --no-via                   deactivate the insertion of a Via-Line\n"
 		"  --password=PASSWORD        password for authentication\n"
 		"                               (if omitted password=username)\n"
 		"  --ignore-redirects         ignore redirects\n"
@@ -105,11 +130,15 @@ void print_long_help() {
 		"  --extract-ip               extract IP from the warning in reply\n"
 		"  --replace-string=STRING    replacement for a special mark in the message\n"
 		"  --replace                  activates replacement of variables\n"
-		"  --nagios-code              returns exit codes Nagios compliant\n"
+		);
+	printf("  --nagios-code              returns exit codes Nagios compliant\n"
 		"  --nagios-warn=NUMBER       return Nagios warning if retrans > number\n"
 		"  --message-body=STRING      send a message with string as body\n"
 		"  --disposition=STRING       Content-Disposition value\n"
+		"  --search=REGEXP            search for a RegExp in replies and return error\n"
+		"                             on failfure\n"
 		);
+	exit(0);
 }
 
 /* prints out some usage help and exits */
@@ -119,6 +148,8 @@ void print_help() {
 		"  -h                displays this help message\n"
 		"  -V                prints version string only\n"
 		"  -f FILE           the file which contains the SIP message to send\n"
+		"                      use - for standard input\n"
+		"  -L                de-activate CR (\\r) insertion in files\n"
 		"  -s SIPURI         the destination server uri in form\n"
 		"                      sip:[user@]servername[:port]\n"
 		"  -T                activates the traceroute mode\n"
@@ -150,7 +181,8 @@ void print_help() {
 		"  -n                use IPs instead of FQDN in the Via-Line\n"
 		"  -i                deactivate the insertion of a Via-Line\n"
 		"  -a PASSWORD       password for authentication\n"
-		"                      (if omitted password=username)\n"
+		"                      (if omitted password=\"\")\n"
+		"  -u STRING         Authentication username\n"
 		);
 	printf(
 		"  -d                ignore redirects\n"
@@ -159,9 +191,12 @@ void print_help() {
 		"  -g STRING         replacement for a special mark in the message\n"
 		"  -G                activates replacement of variables\n"
 		"  -N                returns exit codes Nagios compliant\n"
-		"  -W NUMBER         return Nagios warning if retrans > number\n"
+		"  -q STRING         search for a RegExp in replies and return error\n"
+		"                    on failure\n");
+	printf("  -W NUMBER         return Nagios warning if retrans > number\n"
 		"  -B STRING         send a message with string as body\n"
 		"  -O STRING         Content-Disposition value\n"
+		"  -P NUMBER         Number of processes to start\n"
 		);
 		exit(0);
 }
@@ -170,7 +205,7 @@ int main(int argc, char *argv[])
 {
 	FILE	*pf;
 	char	buff[BUFSIZE];
-	int		length, c;
+	int		length, c, i, j;
 	char	*delim, *delim2;
 #ifdef HAVE_GETOPT_LONG
 	int option_index = 0;
@@ -208,8 +243,12 @@ int main(int argc, char *argv[])
 		{"replace", 0, 0, 'G'},
 		{"nagios-code", 0, 0, 'N'},
 		{"nagios-warn", 1, 0, 'W'},
+		{"search", 1, 0, 'q'},
 		{"message-body", 1, 0, 'B'},
 		{"disposition", 1, 0, 'O'},
+		{"processes", 1, 0, 'P'},
+		{"auth-username", 1, 0, 'u'},
+		{"no-crlf", 0, 0, 'L'},
 		{0, 0, 0, 0}
 	};
 #endif
@@ -218,9 +257,10 @@ int main(int argc, char *argv[])
 	numeric=warning_ext=rand_rem=nonce_count=replace_b=invite=message = 0;
 	sleep_ms=empty_contact=nagios_warn = 0;
 	namebeg=nameend=maxforw= -1;
-	via_ins=redirects = 1;
+	via_ins=redirects=crlf=processes  = 1;
 	username=password=replace_str=hostname=contact_uri=mes_body = NULL;
-	con_dis = NULL;
+	con_dis=auth_username = NULL;
+	re = NULL;
 	address = 0;
 	rport = 5060;
 	expires_t = USRLOC_EXP_DEF;
@@ -229,14 +269,17 @@ int main(int argc, char *argv[])
 	memset(ack, 0, BUFSIZE);
 	memset(fqdn, 0, FQDN_SIZE);
 	memset(messusern, 0, FQDN_SIZE);
+	pid_t pid;
+	struct timespec ts;
+	int upp;
 
 	if (argc==1) print_help();
 
 	/* lots of command line switches to handle*/
 #ifdef HAVE_GETOPT_LONG
-	while ((c=getopt_long(argc, argv, "a:B:b:C:c:de:f:Fg:GhH:iIl:m:MnNo:O:p:r:Rs:t:TUvVwW:x:z", l_opts, &option_index)) != EOF){
+	while ((c=getopt_long(argc, argv, "a:B:b:c:C:de:f:Fg:GhH:iIl:Lm:MnNo:O:p:P:q:r:Rs:t:Tu:UvVwW:x:z", l_opts, &option_index)) != EOF){
 #else
-	while ((c=getopt(argc,argv,"a:B:b:C:c:de:f:Fg:GhH:iIl:m:MnNo:O:p:r:Rs:t:TUvVwW:x:z")) != EOF){
+	while ((c=getopt(argc,argv,"a:B:b:c:C:de:f:Fg:GhH:iIl:Lm:MnNo:O:p:P:q:r:Rs:t:Tu:UvVwW:x:z")) != EOF){
 #endif
 		switch(c){
 			case 'a':
@@ -311,22 +354,34 @@ int main(int argc, char *argv[])
 				flood=1;
 				break;
 			case 'f':
-				/* file is opened in binary mode so that the cr-lf is 
-				   preserved */
-				pf = fopen(optarg, "rb");
-				if (!pf){
-					puts("unable to open the file.\n");
-					exit_code(2);
+				if strncmp(optarg, "-", 1) {
+					/* file is opened in binary mode so that the cr-lf is 
+					   preserved */
+					pf = fopen(optarg, "rb");
+					if (!pf){
+						printf("error: unable to open the file '%s'.\n", optarg);
+						exit_code(2);
+					}
+					length  = fread(buff, 1, sizeof(buff), pf);
+					if (length >= sizeof(buff)){
+						printf("error:the file is too big. try files of less "
+							"than %i bytes.\n", BUFSIZE);
+						printf("      or recompile the program with bigger "
+							"BUFSIZE defined.\n");
+						exit_code(2);
+					}
+					fclose(pf);
 				}
-				length  = fread(buff, 1, sizeof(buff), pf);
-				if (length >= sizeof(buff)){
-					printf("error:the file is too big. try files of less "
-						"than %i bytes.\n", BUFSIZE);
-					printf("      or recompile the program with bigger "
-						"BUFSIZE defined.\n");
-					exit_code(2);
+				else {
+					for(i = 0; i < BUFSIZE - 1; i++) {
+						j = getchar();
+						if (j == EOF)
+							break;
+						else
+							buff[i] = j;
+					}
+					length = i;
 				}
-				fclose(pf);
 				buff[length] = '\0';
 				file_b=1;
 				break;
@@ -351,10 +406,13 @@ int main(int argc, char *argv[])
 			case 'l':
 				lport=atoi(optarg);
 				if (!lport) {
-					puts("error: non-numerical local port number");
+					printf("error: non-numerical local port number");
 					exit_code(2);
 				}
 				break;
+			case 'L':
+				crlf=0;
+				break;
 			case 'm':
 				maxforw=atoi(optarg);
 				if (maxforw==-1) {
@@ -392,6 +450,30 @@ int main(int argc, char *argv[])
 			case 'p':
 				address = getaddress(optarg);
 				break;
+		        case 'P':
+				processes=atoi(optarg);
+				if (!processes) {
+					printf("error: non-numerical number of processes\n");
+					exit_code(2);
+				}
+				break;
+			case 'q':
+				if (re) {
+					/* previously allocated -- free */
+					regfree(re);
+				} else {
+					/* never tried -- allocate */
+					re=malloc(sizeof(regex_t));
+				};
+				if (!re) {
+					fprintf(stderr, "Error: can't allocate RE\n");
+					exit_code(2);
+				};
+				if (regcomp(re, optarg, REG_EXTENDED|REG_ICASE|REG_NEWLINE )!=0) {
+					fprintf(stderr, "Error: compiling RE: %s\n", optarg );
+					exit_code(2);
+				};
+				break;
 			case 'r':
 				rport=atoi(optarg);
 				if (!rport) {
@@ -465,6 +547,11 @@ int main(int argc, char *argv[])
 			case 'U':
 				usrloc=1;
 				break;
+			case 'u':
+				auth_username=malloc(strlen(optarg));
+				strncpy(auth_username, optarg, strlen(optarg));
+				*(auth_username+strlen(optarg)) = '\0';
+				break;
 			case 'v':
 				verbose++;
 				break;
@@ -507,6 +594,10 @@ int main(int argc, char *argv[])
 		}
 	}
 
+	/* replace LF with CRLF if we read from a file */
+	if ((file_b) && (crlf)) {
+		insert_cr(buff);
+	}
 	/* lots of conditions to check */
 	if (trace) {
 		if (usrloc || flood || randtrash) {
@@ -649,8 +740,47 @@ int main(int argc, char *argv[])
 	/* this is not a cryptographic random number generator,
 	   but hey this is only a test-tool => should be satisfying*/
 	srand(time(0));
+	
+	if (processes > 1) {
+		if (signal(SIGCHLD , sigchld_handler)  == SIG_ERR ) {
+			printf("error: Could not install SIGCHLD handler\n");
+			exit_code(2);
+		}
+	}
+
+	for(i = 0; i < processes - 1; i++) {
+		if ((pid = fork()) < 0) {
+			printf("error: Cannot fork\n");
+			exit_code(2);
+		}
+		
+		if (pid == 0){
+	    	/* child */
+			upp = (nameend - namebeg + 1) / processes;
+			namebeg = namebeg + upp * i;
+			nameend = namebeg + upp;
+			shoot(buff);
+		} else {
+			if (lport) {
+				lport++;
+			}
+		}
+		
+		/* Delay execution of children so that the
+		 * time of the first transmission gets spread over
+		 * the retransmission interval evenly
+		 */
+		ts.tv_sec = 0;
+		ts.tv_nsec = (float)DEFAULT_TIMEOUT / (float)processes * (float)1000 * (float)1000;
+		nanosleep(&ts, 0);
+	}
 
 	/* here we go...*/
+	if (processes > 1) {
+		upp = (nameend - namebeg + 1) / processes;
+		namebeg = namebeg + upp * i;
+		nameend = namebeg + upp;
+	}
 	shoot(buff);
 
 	/* normaly we won't come back here, but to satisfy the compiler */
